"""
Vocals SDK Command Line Interface

Provides convenient command-line tools for setup, testing, and demos.
"""

import asyncio
import sys
import os
import logging
from pathlib import Path
from typing import List, Optional, Dict, Any, Union

try:
    import click
except ImportError:
    print("Click is required for CLI functionality. Install with: pip install click")
    sys.exit(1)

from .config import VocalsConfig, get_default_config
from .audio_processor import AudioConfig
from .client import create_vocals


@click.group()
def cli():
    """🎤 Vocals SDK Command Line Interface"""
    pass


@cli.command()
@click.option(
    "--duration", default=30, help="Recording duration in seconds (0 for infinite)"
)
@click.option("--verbose", is_flag=True, help="Enable verbose output")
@click.option("--stats", is_flag=True, default=True, help="Show session statistics")
@click.option("--device", type=int, help="Audio device ID to use")
def demo(duration, verbose, stats, device):
    """Run a microphone streaming demo"""

    async def run_demo():
        print("🎤 Vocals SDK Demo")
        print("=" * 50)

        # Configure logging
        if verbose:
            logging.basicConfig(level=logging.INFO)
        else:
            logging.getLogger("vocals").setLevel(logging.WARNING)

        try:
            # Create SDK with pure default experience - fully self-contained
            sdk = (
                create_vocals()
            )  # No modes, no config changes = full auto-contained experience

            # TODO: Override audio device if specified
            if device is not None:
                print(f"Using audio device ID: {device}")

            print(f"Starting microphone streaming for {duration}s...")
            print("Speak into your microphone!")
            print("Press Ctrl+C to stop early")

            # Stream microphone
            session_stats = await sdk["stream_microphone"](
                duration=duration,
                auto_connect=True,
                auto_playback=True,
                verbose=False,  # SDK handles display automatically
                stats_tracking=stats,
            )

            # Show results
            if stats and session_stats:
                print("\n📊 Session Statistics:")
                print(f"   • Transcriptions: {session_stats.get('transcriptions', 0)}")
                print(f"   • AI Responses: {session_stats.get('responses', 0)}")
                print(
                    f"   • TTS Segments: {session_stats.get('tts_segments_received', 0)}"
                )

        except KeyboardInterrupt:
            print("\n👋 Demo stopped by user")
        except Exception as e:
            print(f"❌ Error: {e}")
        finally:
            if "sdk" in locals():
                try:
                    await sdk["disconnect"]()
                    sdk["cleanup"]()
                except:
                    pass

    asyncio.run(run_demo())


@cli.command()
@click.option("--force", is_flag=True, help="Overwrite existing configuration")
def setup(force):
    """Interactive setup wizard"""

    print("🎤 Vocals SDK Setup Wizard")
    print("=" * 50)

    # Check if .env exists
    env_file = Path(".env")
    if env_file.exists() and not force:
        print("⚠️  .env file already exists. Use --force to overwrite.")
        if not click.confirm("Continue with existing configuration?"):
            return

    # API Key setup
    print("\n1. API Key Configuration")
    api_key = os.environ.get("VOCALS_DEV_API_KEY")
    if api_key:
        print(f"Current API key: {api_key[:10]}...")
        if not click.confirm("Keep current API key?"):
            api_key = None

    if not api_key:
        api_key = click.prompt("Enter your Vocals API key", hide_input=True)
        if not api_key.startswith("vdev_"):
            print("⚠️  API key should start with 'vdev_'")

    # WebSocket endpoint
    print("\n2. WebSocket Configuration")
    current_endpoint = os.environ.get(
        "VOCALS_WS_ENDPOINT", "ws://192.168.1.46:8000/v1/stream/conversation"
    )
    print(f"Current endpoint: {current_endpoint}")

    if click.confirm("Use custom WebSocket endpoint?"):
        endpoint = click.prompt("WebSocket endpoint", default=current_endpoint)
    else:
        endpoint = current_endpoint

    # Audio device selection
    print("\n3. Audio Device Selection")
    try:
        devices = list_audio_devices()
        if devices:
            print("Available audio devices:")
            for device in devices:
                print(
                    f"  {device['id']}: {device['name']} ({device['channels']} channels)"
                )

            device_id = click.prompt(
                "Select device ID (or press Enter for default)",
                type=int,
                default=None,
                show_default=False,
            )
        else:
            device_id = None
    except Exception as e:
        print(f"⚠️  Could not list audio devices: {e}")
        device_id = None

    # Write configuration
    env_content = f"""# Vocals SDK Configuration
VOCALS_DEV_API_KEY={api_key}
VOCALS_WS_ENDPOINT={endpoint}

# Connection settings (auto_connect defaults to False to prevent race conditions)
# VOCALS_AUTO_CONNECT=false
"""

    if device_id is not None:
        env_content += f"VOCALS_AUDIO_DEVICE_ID={device_id}\n"

    with open(".env", "w") as f:
        f.write(env_content)

    print("\n✅ Configuration saved to .env")
    print("You can now run: vocals demo")


@cli.command()
@click.option("--verbose", is_flag=True, help="Show detailed test output")
def test(verbose):
    """Run SDK functionality tests"""

    async def run_tests():
        print("🧪 Vocals SDK Tests")
        print("=" * 50)

        # Configure logging
        if verbose:
            logging.basicConfig(level=logging.DEBUG)
        else:
            logging.getLogger("vocals").setLevel(logging.WARNING)

        tester = VocalsSDKTester()

        # Run tests
        print("Testing connection...")
        await tester.test_connection()

        print("Testing audio recording...")
        await tester.test_audio_recording()

        print("Testing configuration...")
        tester.test_configuration()

        print("Testing audio devices...")
        tester.test_audio_devices()

        # Show results
        tester.print_results()

    asyncio.run(run_tests())


@cli.command()
def devices():
    """List available audio devices"""

    print("🎧 Available Audio Devices")
    print("=" * 50)

    try:
        devices = list_audio_devices()
        if devices:
            for device in devices:
                print(f"ID {device['id']}: {device['name']}")
                print(f"  Input channels: {device['channels']}")
                print(f"  Sample rate: {device['sample_rate']}")
                print()
        else:
            print("No audio input devices found")
    except Exception as e:
        print(f"❌ Error listing devices: {e}")


@cli.command()
@click.argument("device_id", type=int)
@click.option("--duration", default=5, help="Test duration in seconds")
def test_device(device_id, duration):
    """Test a specific audio device"""

    print(f"🎤 Testing Audio Device {device_id}")
    print("=" * 50)

    try:
        print(f"Testing device for {duration} seconds...")
        print("Speak into your microphone!")
        print("Volume meter will show below:")

        test_audio_device(device_id, duration)

    except Exception as e:
        print(f"❌ Error testing device: {e}")


@cli.command()
@click.argument("template_name")
@click.option("--output", "-o", default=None, help="Output file name")
def create_template(template_name, output):
    """Create a template file for quick start

    Available templates:
    - voice_assistant: Simple voice assistant (default experience)
    - file_processor: Process audio files (default experience)
    - conversation_tracker: Track conversations (controlled experience)
    - advanced_voice_assistant: Full control voice assistant (controlled experience)
    """

    templates = get_available_templates()

    if template_name not in templates:
        print(f"❌ Template '{template_name}' not found")
        print(f"Available templates: {', '.join(templates.keys())}")
        return

    if output is None:
        output = f"{template_name}_example.py"

    if Path(output).exists():
        if not click.confirm(f"File {output} exists. Overwrite?"):
            return

    with open(output, "w") as f:
        f.write(templates[template_name])

    print(f"✅ Template created: {output}")


@cli.command()
@click.option(
    "--output", "-o", default="config_report.txt", help="Output file for report"
)
def diagnose(output):
    """Generate diagnostic report"""

    print("🔍 Generating Diagnostic Report")
    print("=" * 50)

    report = generate_diagnostic_report()

    with open(output, "w") as f:
        f.write(report)

    print(f"✅ Diagnostic report saved to: {output}")
    print("\nSummary:")
    print(report.split("\n")[:10])  # Show first 10 lines


# Helper functions
def list_audio_devices():
    """List available audio devices"""
    try:
        import sounddevice as sd

        devices = sd.query_devices()
        result: List[Dict[str, Any]] = []

        for i, device in enumerate(devices):
            # sounddevice returns a DeviceList of dictionaries
            if isinstance(device, dict):
                max_input_channels = device.get("max_input_channels", 0)
                device_name = device.get("name", "Unknown")
                sample_rate = device.get("default_samplerate", 44100)
            else:
                # Fallback for unexpected types
                max_input_channels = 0
                device_name = str(device)
                sample_rate = 44100

            # Only include devices with input channels
            if max_input_channels > 0:
                result.append(
                    {
                        "id": i,
                        "name": str(device_name),
                        "channels": int(max_input_channels),
                        "sample_rate": float(sample_rate),
                    }
                )

        return result
    except ImportError:
        raise Exception("sounddevice not installed")
    except Exception as e:
        raise Exception(f"Error querying audio devices: {e}")


def test_audio_device(device_id: int, duration: int = 5):
    """Test audio device with visual feedback"""
    try:
        import sounddevice as sd
        import numpy as np
        import time

        print(f"Testing device {device_id} for {duration} seconds...")

        def callback(indata, frames, time, status):
            volume = np.sqrt(np.mean(indata**2))
            bar_length = int(volume * 50)
            bar = "█" * bar_length + "░" * (50 - bar_length)
            print(f"\rVolume: [{bar}] {volume:.3f}", end="", flush=True)

        with sd.InputStream(callback=callback, device=device_id, channels=1):
            time.sleep(duration)

        print("\n✅ Device test completed")

    except Exception as e:
        print(f"\n❌ Error testing device: {e}")


def get_available_templates():
    """Get available code templates"""
    return {
        "voice_assistant": '''#!/usr/bin/env python3
"""
Voice Assistant Template
A simple voice assistant using Vocals SDK
"""

import asyncio
import logging
from vocals import create_vocals

# Configure logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

async def voice_assistant():
    """Main voice assistant function"""
    
    # Create SDK instance with default full experience
    # No modes = full auto-contained experience with automatic playback
    sdk = create_vocals()
    
    try:
        print("🎤 Voice Assistant Started")
        print("Speak into your microphone...")
        print("Press Ctrl+C to stop")
        
        # Stream microphone (30 seconds)
        await sdk["stream_microphone"](
            duration=30,
            auto_connect=True,  # Connects automatically since auto_connect defaults to False
            auto_playback=True,  # AI responses play automatically
            verbose=False
        )
        
    except KeyboardInterrupt:
        print("\\n👋 Voice assistant stopped")
    finally:
        await sdk["disconnect"]()
        sdk["cleanup"]()

if __name__ == "__main__":
    asyncio.run(voice_assistant())
''',
        "file_processor": '''#!/usr/bin/env python3
"""
Audio File Processor Template
Process audio files with Vocals SDK
"""

import asyncio
import sys
from pathlib import Path
from vocals import create_vocals

async def process_audio_file(file_path: str):
    """Process an audio file and get AI responses"""
    
    # Check if file exists
    if not Path(file_path).exists():
        print(f"❌ File not found: {file_path}")
        return
    
    # Create SDK instance with default full experience
    # No modes = full auto-contained experience with automatic playback
    sdk = create_vocals()
    
    try:
        print(f"🎵 Processing file: {file_path}")
        
        # Stream audio file
        await sdk["stream_audio_file"](
            file_path=file_path,
            verbose=False,
            auto_connect=True  # Connects automatically since auto_connect defaults to False
        )
        
        # Wait for TTS playback to complete (playback is automatic in default mode)
        print("⏳ Waiting for AI response playback...")
        while sdk["get_is_playing"]():
            await asyncio.sleep(0.1)
        
        print("✅ File processing completed")
        
    except Exception as e:
        print(f"❌ Error processing file: {e}")
    finally:
        await sdk["disconnect"]()
        sdk["cleanup"]()

if __name__ == "__main__":
    if len(sys.argv) != 2:
        print("Usage: python file_processor.py <audio_file.wav>")
        sys.exit(1)
    
    file_path = sys.argv[1]
    asyncio.run(process_audio_file(file_path))
''',
        "conversation_tracker": '''#!/usr/bin/env python3
"""
Conversation Tracker Template
Track and analyze conversation with AI
"""

import asyncio
from vocals import (
    create_vocals,
    create_conversation_tracker,
)

async def conversation_session():
    """Run a conversation session with tracking"""
    
    # Create SDK with controlled experience for custom tracking
    # Using modes disables automatic handlers - we implement custom ones
    sdk = create_vocals(modes=['transcription', 'voice_assistant'])
    tracker = create_conversation_tracker()
    
    # Custom tracking handler with manual playback control
    def track_conversation(message):
        # Custom display logic
        if message.type == "transcription" and message.data:
            text = message.data.get("text", "")
            is_partial = message.data.get("is_partial", False)
            if not is_partial and text:
                print(f"🎤 You: {text}")
                
        elif message.type == "llm_response" and message.data:
            response = message.data.get("response", "")
            if response:
                print(f"🤖 AI: {response}")
                
        elif message.type == "tts_audio" and message.data:
            text = message.data.get("text", "")
            if text:
                print(f"🔊 Playing: {text}")
                # Manually start playback since auto_playback=False
                asyncio.create_task(sdk["play_audio"]())
        
        # Track conversation for analysis
        if message.type == "transcription" and message.data:
            text = message.data.get("text", "")
            is_partial = message.data.get("is_partial", False)
            if text:
                tracker["add_transcription"](text, is_partial)
                
        elif message.type == "llm_response" and message.data:
            response = message.data.get("response", "")
            if response:
                tracker["add_response"](response)
    
    # Register our custom handler
    sdk["on_message"](track_conversation)
    
    try:
        print("💬 Conversation Session Started")
        print("Have a conversation with the AI...")
        print("Press Ctrl+C to stop and see analysis")
        
        # Run conversation with manual playback control
        await sdk["stream_microphone"](
            duration=60,  # 1 minute
            auto_connect=True,  # Connects automatically since auto_connect defaults to False
            auto_playback=False,  # We control playback manually via our handler
            verbose=False
        )
        
    except KeyboardInterrupt:
        print("\\n📊 Conversation Analysis:")
        
    finally:
        # Print conversation history
        tracker["print_conversation"]()
        
        # Print statistics
        stats = tracker["get_stats"]()
        print(f"\\n📈 Session Statistics:")
        print(f"   Duration: {stats['duration']:.1f} seconds")
        print(f"   Exchanges: {stats['transcriptions'] + stats['responses']}")
        
        await sdk["disconnect"]()
        sdk["cleanup"]()

if __name__ == "__main__":
    asyncio.run(conversation_session())
''',
        "advanced_voice_assistant": '''#!/usr/bin/env python3
"""
Advanced Voice Assistant Template
Full control over voice assistant behavior using controlled mode

This template demonstrates two approaches for handling audio:
1. Built-in audio playback (default) - uses SDK's built-in audio player
2. Custom audio processing - process audio segments with your own function

To switch between approaches, comment/uncomment the relevant sections
in the tts_audio message handler.
"""

import asyncio
import logging
from vocals import create_vocals

# Configure logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

async def advanced_voice_assistant():
    """Advanced voice assistant with full manual control"""
    
    # Create SDK with specific modes for controlled experience
    # Using modes disables all automatic handlers - we have complete control
    # This prevents duplicate message processing that would occur with default handlers
    sdk = create_vocals(modes=['transcription', 'voice_assistant'])
    
    # Custom state tracking for conversation flow
    conversation_state = {
        'listening': False,
        'processing': False,
        'speaking': False
    }
    
    def handle_messages(message):
        """Custom message handler with complete control over all behavior"""
        
        if message.type == "transcription" and message.data:
            text = message.data.get("text", "")
            is_partial = message.data.get("is_partial", False)
            
            if is_partial:
                # Show live transcription updates
                print(f"\\r🎤 Listening: {text}...", end="", flush=True)
                conversation_state['listening'] = True
            else:
                # Final transcription result
                print(f"\\n✅ You said: {text}")
                conversation_state['listening'] = False
                conversation_state['processing'] = True
                
        elif message.type == "llm_response_streaming" and message.data:
            token = message.data.get("token", "")
            is_complete = message.data.get("is_complete", False)
            
            if not conversation_state['processing']:
                print("\\n💭 AI Thinking: ", end="", flush=True)
                conversation_state['processing'] = True
                
            if token:
                print(token, end="", flush=True)
                
            if is_complete:
                print()  # New line
                conversation_state['processing'] = False
                
        elif message.type == "tts_audio" and message.data:
            text = message.data.get("text", "")
            if text and not conversation_state['speaking']:
                print(f"🔊 AI speaking: {text}")
                conversation_state['speaking'] = True
                
                # OPTION 1: Use built-in audio playback (current approach)
                # Manually trigger playback since auto_playback=False
                asyncio.create_task(sdk["play_audio"]())
                
                # OPTION 2: Process audio queue with custom handler (new approach)
                # Uncomment the lines below to use custom audio processing instead
                # def my_custom_audio_handler(segment):
                #     print(f"🎵 Custom processing: {segment.text}")
                #     # Here you can:
                #     # - Save audio to file
                #     # - Send to external audio player
                #     # - Convert audio format
                #     # - Analyze audio data
                #     # - Or any other custom processing
                #     
                #     # Example: Save to file
                #     # import base64
                #     # audio_data = base64.b64decode(segment.audio_data)
                #     # filename = f"audio_{segment.segment_id}.wav"
                #     # with open(filename, 'wb') as f:
                #     #     f.write(audio_data)
                #     # print(f"💾 Saved to: {filename}")
                #     pass
                # 
                # # Process audio queue with custom handler
                # processed_count = sdk["process_audio_queue"](
                #     my_custom_audio_handler, 
                #     consume_all=True
                # )
                # print(f"✅ Processed {processed_count} audio segments")
                
        elif message.type == "speech_interruption":
            print("\\n🛑 Speech interrupted")
            conversation_state['speaking'] = False
    
    # Register our custom message handler
    sdk["on_message"](handle_messages)
    
    # Custom connection handler
    def handle_connection(state):
        if state.name == "CONNECTED":
            print("✅ Connected to voice assistant")
        elif state.name == "DISCONNECTED":
            print("❌ Disconnected from voice assistant")
    
    sdk["on_connection_change"](handle_connection)
    
    try:
        print("🎤 Advanced Voice Assistant Started")
        print("Full control mode - custom handlers active")
        print("Features: live transcription, streaming responses, manual playback")
        print("Speak into your microphone...")
        print("Press Ctrl+C to stop")
        
        # Stream microphone with complete manual control
        await sdk["stream_microphone"](
            duration=0,  # Infinite recording
            auto_connect=True,  # Connects automatically since auto_connect defaults to False
            auto_playback=False,  # We have complete manual control over playback
            verbose=False
        )
        
    except KeyboardInterrupt:
        print("\\n👋 Advanced voice assistant stopped")
    finally:
        await sdk["disconnect"]()
        sdk["cleanup"]()

if __name__ == "__main__":
    asyncio.run(advanced_voice_assistant())
''',
    }


def generate_diagnostic_report():
    """Generate a diagnostic report"""
    import platform
    import sys

    report = f"""Vocals SDK Diagnostic Report
Generated: {__import__('datetime').datetime.now()}

System Information:
- Platform: {platform.platform()}
- Python Version: {sys.version}
- Python Path: {sys.executable}

Environment Variables:
- VOCALS_DEV_API_KEY: {'SET' if os.environ.get('VOCALS_DEV_API_KEY') else 'NOT SET'}
- VOCALS_WS_ENDPOINT: {os.environ.get('VOCALS_WS_ENDPOINT', 'NOT SET')}

Dependencies:
"""

    # Check dependencies
    deps_to_check = [
        "sounddevice",
        "numpy",
        "websockets",
        "aiohttp",
        "PyJWT",
        "python-dotenv",
        "click",
    ]

    for dep in deps_to_check:
        try:
            __import__(dep)
            report += f"- {dep}: ✅ Installed\n"
        except ImportError:
            report += f"- {dep}: ❌ Missing\n"

    # Check audio devices
    report += "\nAudio Devices:\n"
    try:
        devices = list_audio_devices()
        for device in devices:
            report += (
                f"- {device['id']}: {device['name']} ({device['channels']} channels)\n"
            )
    except Exception as e:
        report += f"- Error: {e}\n"

    return report


class VocalsSDKTester:
    """Test framework for SDK functionality"""

    def __init__(self):
        self.results = []

    async def test_connection(self):
        """Test WebSocket connection"""
        try:
            from .client import create_vocals

            sdk = create_vocals()
            await sdk["connect"]()
            connected = sdk["get_is_connected"]()
            await sdk["disconnect"]()

            self.results.append(
                ("WebSocket Connection", "✅ PASS" if connected else "❌ FAIL")
            )
            return connected
        except Exception as e:
            self.results.append(("WebSocket Connection", f"❌ ERROR: {e}"))
            return False

    async def test_audio_recording(self):
        """Test audio recording functionality"""
        try:
            from .client import create_vocals

            sdk = create_vocals()
            await sdk["start_recording"]()
            await asyncio.sleep(0.5)
            recording = sdk["get_is_recording"]()
            await sdk["stop_recording"]()

            self.results.append(
                ("Audio Recording", "✅ PASS" if recording else "❌ FAIL")
            )
            return recording
        except Exception as e:
            self.results.append(("Audio Recording", f"❌ ERROR: {e}"))
            return False

    def test_configuration(self):
        """Test configuration"""
        try:
            issues = validate_config()
            if not issues:
                self.results.append(("Configuration", "✅ PASS"))
                return True
            else:
                self.results.append(
                    ("Configuration", f"❌ ISSUES: {', '.join(issues)}")
                )
                return False
        except Exception as e:
            self.results.append(("Configuration", f"❌ ERROR: {e}"))
            return False

    def test_audio_devices(self):
        """Test audio device availability"""
        try:
            devices = list_audio_devices()
            if devices:
                self.results.append(
                    ("Audio Devices", f"✅ {len(devices)} devices found")
                )
                return True
            else:
                self.results.append(("Audio Devices", "❌ No input devices found"))
                return False
        except Exception as e:
            self.results.append(("Audio Devices", f"❌ ERROR: {e}"))
            return False

    def print_results(self):
        """Print test results"""
        print("\n🧪 Test Results:")
        print("=" * 50)
        for test, result in self.results:
            print(f"{test}: {result}")


def validate_config() -> List[str]:
    """Validate configuration and return issues"""
    issues = []

    # Check API key
    api_key = os.environ.get("VOCALS_DEV_API_KEY")
    if not api_key:
        issues.append("VOCALS_DEV_API_KEY not set")
    elif not api_key.startswith("vdev_"):
        issues.append("Invalid API key format")

    # Check audio system
    try:
        import sounddevice as sd

        sd.query_devices()
    except ImportError:
        issues.append("sounddevice not installed")
    except Exception as e:
        issues.append(f"Audio system error: {e}")

    # Check WebSocket endpoint
    endpoint = os.environ.get("VOCALS_WS_ENDPOINT")
    if endpoint and not endpoint.startswith("ws"):
        issues.append("Invalid WebSocket endpoint format")

    return issues


if __name__ == "__main__":
    cli()
